import time
import urllib.parse as urllib

from datetime import datetime
from calendar import monthrange
from netforce.model import Model, fields, get_model
from netforce.access import get_active_company

from . import utils

class ReportHDCaseSummary(Model):
    _name="clinic.report.hd.case.summary"
    _string="Hemodialysis Report Summary"
    _transient=True
    
    _fields={
        "date": fields.Date("Month", required=True),
    }

    _defaults={
        'date': lambda *a: time.strftime("%Y-%m-%d"),        
    }
    
    def get_report_data(self,ids,context={}):
        company_id=get_active_company()
        company=get_model("company").browse(company_id)

        date=datetime.now().strftime("%Y-%m-%d")
        if ids:
            obj=self.browse(ids)[0]
            date=obj.date 
        year=int(date[0:4])
        prev_year=year
        next_year=year
        crr_month=int(date[5:7])
        prev_month=crr_month-1
        next_month=crr_month+1
        if crr_month==1:
            prev_month=12
            prev_year-=1
        if crr_month==12:
            next_month=1
            next_year+=1

        month_str=utils.MONTHS['th_TH'][crr_month]
        next_month_str=utils.MONTHS['th_TH'][next_month]
        prev_month_str=utils.MONTHS['th_TH'][prev_month]
              
        def encode_url(dom):
            dom='%s'%dom
            dom=urllib.quote(dom.encode('utf-8'))
            return dom

        def replace_quote(dom=""):
            return dom.replace("'","\"")
            

        # number of hd case of this month
        dom=[]
        weekday, crr_total_day=monthrange(year, crr_month)
        time_start='%s-%s-01 00:00:00'%(year,str(crr_month).zfill(2))
        time_stop='%s-%s-%s 23:59:59'%(year,str(crr_month).zfill(2),crr_total_day)
        dom.append(["time_start",">=",time_start])
        dom.append(["time_stop","<=",time_stop])
        dom.append(["state","=","completed"])
        crr_total=len(get_model("clinic.hd.case").search(dom))
        items={}
        items['topic1']={
                'month': month_str,
                'qty': crr_total or 0,
                'action': 'clinic_hd_case',
                'action_options': 'mode=list&search_domain=%s&tab_no=4'%replace_quote('%s'%dom),
            }

        # number of patient from previous
        dom=[]
        weekday, prev_total_day=monthrange(prev_year, prev_month)
        time_stop='%s-%s-%s'%(prev_year,str(prev_month).zfill(2),prev_total_day)
        dom.append(['reg_date','<=',time_stop])
        npatient=len(get_model("clinic.patient").search(dom))
        dom=replace_quote('%s'%dom)
        items['topic2']={
                'month': prev_month_str,
                'qty': npatient or 0,
                'action': 'clinic_patient',
                'action_options': 'mode=list&search_domain=%s'%dom,
            }

        # new patient of this month
        dom=[]
        weekday, crr_total_day=monthrange(prev_year, crr_month)
        time_start='%s-%s-01'%(year,str(crr_month).zfill(2))
        time_stop='%s-%s-%s'%(year,str(crr_month).zfill(2),crr_total_day)
        dom.append(['reg_date','>=',time_start])
        dom.append(['reg_date','<=',time_stop])
        new_patients=get_model('clinic.patient').search_browse(dom)
        dom=replace_quote('%s'%dom)
        items['topic3']={
                'month': month_str,
                'qty': len(new_patients) or 0,
                'action': 'clinic_patient',
                'action_options': 'mode=list&search_domain=%s'%dom,
            }

        # number for patient who resign for this month
        dom=[]
        time_start='%s-%s-01'%(year,str(crr_month).zfill(2))
        time_stop='%s-%s-%s'%(year,str(crr_month).zfill(2),crr_total_day)
        dom.append(['resign_date','>=',time_start])
        dom.append(['resign_date','<=',time_stop])
        dom.append(['active','=',False])
        resign_patients=get_model('clinic.patient').search_browse(dom)
        del dom[-1]
        dom=replace_quote('%s'%dom)
        items['topic4']={
                'month': month_str,
                'qty': len(resign_patients) or 0,
                'action': 'clinic_patient',
                'action_options': 'mode=list&search_domain=%s&tab_no=1'%dom,
            }

        # all patient who are in hospital on select month
        dom=[]
        weekday, crr_total_day=monthrange(year, crr_month)
        time_stop='%s-%s-%s'%(year,str(crr_month).zfill(2),crr_total_day)
        dom.append(['reg_date','<=',time_stop])
        total_patient=get_model('clinic.patient').search_browse(dom)
        dom=replace_quote('%s'%dom)
        items['topic5']={
                'month': next_month_str,
                'qty': len(total_patient) or 0,
                'action': 'clinic_patient',
                'action_options': 'mode=list&search_domain=%s'%dom,
            }

        # all patient who are sc in hospital on select month
        dom=[]
        time_start='%s-%s-01'%(year,str(crr_month).zfill(2))
        time_stop='%s-%s-%s'%(year,str(crr_month).zfill(2),crr_total_day)
        dom.append(['reg_date','>=',time_start])
        dom.append(['reg_date','<=',time_stop])
        dom.append(['type','=','sc'])
        npatients=get_model("clinic.patient").search(dom)
        dom=replace_quote('%s'%dom)
        items['topic6']={
                'month': '',
                'qty': len(npatients),
                'action': 'clinic_patient',
                'action_options': 'mode=list&search_domain=%s&tab_no=2'%dom,
            }

        # all patient who are uc in hospital on select month
        dom=[]
        time_start='%s-%s-01'%(year,str(crr_month).zfill(2))
        time_stop='%s-%s-%s'%(year,str(crr_month).zfill(2),crr_total_day)
        dom.append(['reg_date','>=',time_start])
        dom.append(['reg_date','<=',time_stop])
        dom.append(['type','=','uc'])
        npatients=get_model("clinic.patient").search(dom)
        dom=replace_quote('%s'%dom)
        items['topic7']={
                'month':'',
                'qty': len(npatients),
                'action': 'clinic_patient',
                'action_options': 'mode=list&search_domain=%s&tab_no=3'%dom,
            }

        # all patient who are other in hospital on select month
        dom=[]
        time_start='%s-%s-01'%(year,str(crr_month).zfill(2))
        time_stop='%s-%s-%s'%(year,str(crr_month).zfill(2),crr_total_day)
        dom.append(['reg_date','>=',time_start])
        dom.append(['reg_date','<=',time_stop])
        dom.append(['type','=','others'])
        npatients=get_model("clinic.patient").search(dom)
        dom=replace_quote('%s'%dom)
        items['topic8']={
                'month': '',
                'qty': len(npatients),
                'action': 'clinic_patient',
                'action_options': 'mode=list&search_domain=%s&tab_no=4'%dom,
        }
        lines=[] 
        index=1
        for item in items:
            topic='topic%s'%index
            name=utils.TOPICS[topic]['name']
            unit=utils.TOPICS[topic]['unit']
            line=items[topic]
            line['topic']=name
            line['unit']=unit
            lines.append(line)
            index+=1

        context['defaults']={'date': date}

        medicals=get_model("clinic.report.medical.summary").get_report_data(ids=[],context=context)
        year=year+543
        data={
            'date': date,
            'month': month_str,
            'year': year,
            'lines': lines,
            'recent_patients': get_model("clinic.report.recent.patient").get_report_data(ids=[],context=context)['lines'],
            'resign_patients': get_model("clinic.report.discontinue.patient").get_report_data(ids=[],context=context)['lines'],
            'medicals': medicals['lines'],
            'titles': medicals['titles'],
            'company_name': company.name or "",
            'parent_company_name': company.parent_id.name or "",
        }
        return data

ReportHDCaseSummary.register()
